package schedframe.resources.computing.profiles.energy.power.ui;



import java.util.List;


import schedframe.resources.ResourceStatus;
import schedframe.resources.computing.ComputingResource;
import schedframe.resources.computing.Node;
import schedframe.resources.computing.profiles.energy.ResourceEvent;
import schedframe.resources.computing.profiles.energy.ResourceEventType;
import schedframe.resources.computing.profiles.energy.power.PowerProfile;
import schedframe.resources.computing.profiles.energy.power.PowerState;
import schedframe.resources.computing.profiles.energy.power.PowerStateName;
import schedframe.resources.computing.profiles.energy.power.StandardPowerStateName;
import schedframe.resources.computing.profiles.energy.power.Transition;
import simulator.DataCenterWorkloadSimulator;

public class NodePowerInterface extends ComputingResourcePowerInterface{

	public NodePowerInterface(ComputingResource resource, PowerProfile pp){
		super(resource, pp);
		currentPowerState = StandardPowerStateName.ON;
	}

	public boolean setPowerState(PowerStateName state){
		if(!supportPowerState(state))
			return false;
		if(state != currentPowerState){
			currentPowerState = state;
			
			Node node = (Node) resource;
			for(ComputingResource child: node.getProcessors()){
				if(child.getPowerInterface() != null){
					child.getPowerInterface().setPowerState(state);	
				}
			}

			if(state == StandardPowerStateName.OFF || state == StandardPowerStateName.POWEROFF || state == StandardPowerStateName.BOOT){
				node.setStatus(ResourceStatus.UNAVAILABLE);
			}
			else if(state == StandardPowerStateName.ON ){
				node.setStatus(ResourceStatus.FREE);
			}
	
			node.handleEvent(new ResourceEvent(ResourceEventType.POWER_STATE_CHANGED, node.getFullName()));

			//node.handleEvent(new EnergyEvent(EnergyEventType.POWER_STATE_CHANGED, computingNode.getName()));
			//ResourceController.traceResource(new DateTime().getMillis(), resource.getFullName(), "POWER_STATE_CHANGED", state.getName());	
		}
		return true;
	}

	public void turnOn(){
		/*double timePenalty = getTransitionTime(currentPowerState, StandardPowerStateName.ON);
		setPowerState(StandardPowerStateName.BOOT);
		DataCenterWorkloadSimulator.getEventManager().sendToResource(resource.getFullName(), timePenalty, new ResourceEvent(ResourceEventType.UPDATE_POWER_STATE, StandardPowerStateName.ON, resource.getFullName()));
		 */		
		setPowerState(StandardPowerStateName.ON);
	}
	
	public void turnOff(){
		/*double timePenalty = getTransitionTime(currentPowerState, StandardPowerStateName.OFF);
		setPowerState(StandardPowerStateName.POWEROFF);
		DataCenterWorkloadSimulator.getEventManager().sendToResource(resource.getFullName(), timePenalty, new ResourceEvent(ResourceEventType.UPDATE_POWER_STATE, StandardPowerStateName.OFF, resource.getFullName()));*/
		setPowerState(StandardPowerStateName.OFF);
	}

	private double getTransitionTime(PowerStateName from, PowerStateName to){
		List<PowerState> availablePowerStates = getSupportedPowerStates();
		double time = 0;
		for(PowerState ps : availablePowerStates){
			if(ps.getName().getLabel().equals(from.getLabel())){
				List<Transition> transitions = ps.getTransitions();
				for(Transition t: transitions){
					if(t.getTo().getLabel().equals(to)){
						time = t.getTime();
					}
				}
			}
		}
		return time;
	}


}